//class QuizServer provides quiz questions to remote Quiz clients

import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.*;  //for class Vector

public class QuizServer extends Frame {
   TextArea display;  //GUI display for QuizServer
   static int QuizNumber;    //Which quiz number is this?  From command-line argument  
   static Vector quizLines;  //Quiz questions from a file
   static DataInputStream quizInput;  //File providing quizLines questions
   static QuizScoresFile studentScores = new QuizScoresFile();  //Database of students
   static ServerSocket server;  //Network server for QuizClients
  
   public static  void readQuiz(String filename)
   { //read quiz from filename into quizLInes
      String line = new String("");
      quizLines = new Vector(50);
      quizLines.addElement(line);      
      try //open file and read questions
      { quizInput = new DataInputStream( new FileInputStream(filename) );
         while (!line.equals("#"))
         { line = quizInput.readLine();
           quizLines.addElement(line);
         }
       }
       catch ( EOFException e ) 
       { System.err.println( "Unexpected EOF in " + filename + "\n"+ e.toString() );
          System.exit(1);
       }  
       catch ( FileNotFoundException e )
       { System.err.println( "Could not open " + filename + "\n"+ e.toString() );
          System.exit(1);
       }
       catch ( IOException e )
       { System.err.println("Error while server is reading " + filename + "\n" + e.toString() );
          System.exit(1);
       }      
    }             
    
   public QuizServer()
   {
      super( "Server" );
      display = new TextArea( "", 0, 0,
                    TextArea.SCROLLBARS_VERTICAL_ONLY );
      add( display, BorderLayout.CENTER );
      setSize( 300, 150 );
      setVisible( true );
    
      //Create a ServerSocket
      try
      { server = new ServerSocket( 5000, 100 ); 
      }
      catch (IOException e)
      { System.out.println("Couldn't create ServerSocket for QuizServer"); 
         e.printStackTrace(); 
      }
   }
 
   public static void main( String args[] )
   {
     //Check command-line parameter (should be quiz #)
     if (args.length < 1 || args.length > 1)
     { System.out.println("Usage: QuizServer <number>");
        System.exit(1);
     } //OK, get quiz number from command-line
     QuizNumber = new Integer(args[0].trim()).intValue() - 1;  //Convert to index, starting with 0
     if (QuizNumber < 0 || QuizNumber > 15)
     { System.out.println("Usage: QuizServer <number> (between 1 and 16)");
        System.exit(1);
     } //OK, read in the quiz file
     readQuiz("UMquiz"+args[0]+".txt");  //read quiz file
     
     //Set up a Frame for the QuizServer
     QuizServer qs = new QuizServer();
     qs.addWindowListener( new CloseWindowAndExit() );
     
      //Read database file of student names and scores
      studentScores = studentScores.readFile(studentScores);   
      //Mark this quiz as taken (in "newbie" line)
      studentScores.addScore( 0,QuizNumber,1 );
 
      //Wait for connections from students running the Quiz program     
      QuizClient client;
      Vector clients = new Vector(25);
      while (true)
         try 
         { client =  new QuizClient( server.accept(),  qs);
           client.start();
           clients.addElement(client);           
         }
         catch (IOException e)
         { System.out.println("QuizServer couldn't accept QuizClient connection"); 
            e.printStackTrace(); 
         }
    } //main()
   
} //class QuizServer
